@@ -1,12 +1,18 @@
 /*
     FUSE: Filesystem in Userspace
-    Copyright (C) 2001-2005  Miklos Szeredi <miklos@szeredi.hu>
+    Copyright (C) 2001-2007  Miklos Szeredi <miklos@szeredi.hu>
 
     This program can be distributed under the terms of the GNU GPL.
     See the file COPYING.
+
+    gcc -Wall `pkg-config fuse --cflags --libs` fusexmp.c -o fusexmp
 */
 
+#define FUSE_USE_VERSION 26
+
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
 
 #ifdef linux
 /* For pread()/pwrite() */
@@ -20,7 +26,7 @@
 #include <fcntl.h>
 #include <dirent.h>
 #include <errno.h>
-#include <sys/statfs.h>
+#include <sys/time.h>
 #ifdef HAVE_SETXATTR
 #include <sys/xattr.h>
 #endif
@@ -30,7 +36,18 @@ static int xmp_getattr(const char *path, struct stat *stbuf)
     int res;
 
     res = lstat(path, stbuf);
-    if(res == -1)
+    if (res == -1)
+        return -errno;
+
+    return 0;
+}
+
+static int xmp_access(const char *path, int mask)
+{
+    int res;
+
+    res = access(path, mask);
+    if (res == -1)
         return -errno;
 
     return 0;
@@ -41,7 +58,7 @@ static int xmp_readlink(const char *path, char *buf, size_t size)
     int res;
 
     res = readlink(path, buf, size - 1);
-    if(res == -1)
+    if (res == -1)
         return -errno;
 
     buf[res] = '\0';
@@ -59,10 +76,10 @@ static int xmp_readdir(const char *path, void *buf, fuse_fill_dir_t filler,
     (void) fi;
 
     dp = opendir(path);
-    if(dp == NULL)
+    if (dp == NULL)
         return -errno;
 
-    while((de = readdir(dp)) != NULL) {
+    while ((de = readdir(dp)) != NULL) {
         struct stat st;
         memset(&st, 0, sizeof(st));
         st.st_ino = de->d_ino;
@@ -79,8 +96,17 @@ static int xmp_mknod(const char *path, mode_t mode, dev_t rdev)
 {
     int res;
 
-    res = mknod(path, mode, rdev);
-    if(res == -1)
+    /* On Linux this could just be 'mknod(path, mode, rdev)' but this
+       is more portable */
+    if (S_ISREG(mode)) {
+        res = open(path, O_CREAT | O_EXCL | O_WRONLY, mode);
+        if (res >= 0)
+            res = close(res);
+    } else if (S_ISFIFO(mode))
+        res = mkfifo(path, mode);
+    else
+        res = mknod(path, mode, rdev);
+    if (res == -1)
         return -errno;
 
     return 0;
@@ -91,7 +117,7 @@ static int xmp_mkdir(const char *path, mode_t mode)
     int res;
 
     res = mkdir(path, mode);
-    if(res == -1)
+    if (res == -1)
         return -errno;
 
     return 0;
@@ -102,7 +128,7 @@ static int xmp_unlink(const char *path)
     int res;
 
     res = unlink(path);
-    if(res == -1)
+    if (res == -1)
         return -errno;
 
     return 0;
@@ -113,7 +139,7 @@ static int xmp_rmdir(const char *path)
     int res;
 
     res = rmdir(path);
-    if(res == -1)
+    if (res == -1)
         return -errno;
 
     return 0;
@@ -124,7 +150,7 @@ static int xmp_symlink(const char *from, const char *to)
     int res;
 
     res = symlink(from, to);
-    if(res == -1)
+    if (res == -1)
         return -errno;
 
     return 0;
@@ -135,7 +161,7 @@ static int xmp_rename(const char *from, const char *to)
     int res;
 
     res = rename(from, to);
-    if(res == -1)
+    if (res == -1)
         return -errno;
 
     return 0;
@@ -146,7 +172,7 @@ static int xmp_link(const char *from, const char *to)
     int res;
 
     res = link(from, to);
-    if(res == -1)
+    if (res == -1)
         return -errno;
 
     return 0;
@@ -157,7 +183,7 @@ static int xmp_chmod(const char *path, mode_t mode)
     int res;
 
     res = chmod(path, mode);
-    if(res == -1)
+    if (res == -1)
         return -errno;
 
     return 0;
@@ -168,7 +194,7 @@ static int xmp_chown(const char *path, uid_t uid, gid_t gid)
     int res;
 
     res = lchown(path, uid, gid);
-    if(res == -1)
+    if (res == -1)
         return -errno;
 
     return 0;
@@ -179,30 +205,35 @@ static int xmp_truncate(const char *path, off_t size)
     int res;
 
     res = truncate(path, size);
-    if(res == -1)
+    if (res == -1)
         return -errno;
 
     return 0;
 }
 
-static int xmp_utime(const char *path, struct utimbuf *buf)
+static int xmp_utimens(const char *path, const struct timespec ts[2])
 {
     int res;
+    struct timeval tv[2];
 
-    res = utime(path, buf);
-    if(res == -1)
+    tv[0].tv_sec = ts[0].tv_sec;
+    tv[0].tv_usec = ts[0].tv_nsec / 1000;
+    tv[1].tv_sec = ts[1].tv_sec;
+    tv[1].tv_usec = ts[1].tv_nsec / 1000;
+
+    res = utimes(path, tv);
+    if (res == -1)
         return -errno;
 
     return 0;
 }
 
-
 static int xmp_open(const char *path, struct fuse_file_info *fi)
 {
     int res;
 
     res = open(path, fi->flags);
-    if(res == -1)
+    if (res == -1)
         return -errno;
 
     close(res);
@@ -217,11 +248,11 @@ static int xmp_read(const char *path, char *buf, size_t size, off_t offset,
 
     (void) fi;
     fd = open(path, O_RDONLY);
-    if(fd == -1)
+    if (fd == -1)
         return -errno;
 
     res = pread(fd, buf, size, offset);
-    if(res == -1)
+    if (res == -1)
         res = -errno;
 
     close(fd);
@@ -236,23 +267,23 @@ static int xmp_write(const char *path, const char *buf, size_t size,
 
     (void) fi;
     fd = open(path, O_WRONLY);
-    if(fd == -1)
+    if (fd == -1)
         return -errno;
 
     res = pwrite(fd, buf, size, offset);
-    if(res == -1)
+    if (res == -1)
         res = -errno;
 
     close(fd);
     return res;
 }
 
-static int xmp_statfs(const char *path, struct statfs *stbuf)
+static int xmp_statfs(const char *path, struct statvfs *stbuf)
 {
     int res;
 
-    res = statfs(path, stbuf);
-    if(res == -1)
+    res = statvfs(path, stbuf);
+    if (res == -1)
         return -errno;
 
     return 0;
@@ -286,7 +317,7 @@ static int xmp_setxattr(const char *path, const char *name, const char *value,
                         size_t size, int flags)
 {
     int res = lsetxattr(path, name, value, size, flags);
-    if(res == -1)
+    if (res == -1)
         return -errno;
     return 0;
 }
@@ -295,7 +326,7 @@ static int xmp_getxattr(const char *path, const char *name, char *value,
                     size_t size)
 {
     int res = lgetxattr(path, name, value, size);
-    if(res == -1)
+    if (res == -1)
         return -errno;
     return res;
 }
@@ -303,7 +334,7 @@ static int xmp_getxattr(const char *path, const char *name, char *value,
 static int xmp_listxattr(const char *path, char *list, size_t size)
 {
     int res = llistxattr(path, list, size);
-    if(res == -1)
+    if (res == -1)
         return -errno;
     return res;
 }
@@ -311,7 +342,7 @@ static int xmp_listxattr(const char *path, char *list, size_t size)
 static int xmp_removexattr(const char *path, const char *name)
 {
     int res = lremovexattr(path, name);
-    if(res == -1)
+    if (res == -1)
         return -errno;
     return 0;
 }
@@ -319,6 +350,7 @@ static int xmp_removexattr(const char *path, const char *name)
 
 static struct fuse_operations xmp_oper = {
     .getattr	= xmp_getattr,
+    .access	= xmp_access,
     .readlink	= xmp_readlink,
     .readdir	= xmp_readdir,
     .mknod	= xmp_mknod,
@@ -331,7 +363,7 @@ static struct fuse_operations xmp_oper = {
     .chmod	= xmp_chmod,
     .chown	= xmp_chown,
     .truncate	= xmp_truncate,
-    .utime	= xmp_utime,
+    .utimens	= xmp_utimens,
     .open	= xmp_open,
     .read	= xmp_read,
     .write	= xmp_write,
@@ -349,5 +381,5 @@ static struct fuse_operations xmp_oper = {
 int main(int argc, char *argv[])
 {
     umask(0);
-    return fuse_main(argc, argv, &xmp_oper);
+    return fuse_main(argc, argv, &xmp_oper, NULL);
 }
