@@ -1,6 +1,6 @@
 /*
     FUSE: Filesystem in Userspace
-    Copyright (C) 2001-2006  Miklos Szeredi <miklos@szeredi.hu>
+    Copyright (C) 2001-2007  Miklos Szeredi <miklos@szeredi.hu>
 
     This program can be distributed under the terms of the GNU GPL.
     See the file COPYING.
@@ -24,16 +24,14 @@ static int hello_getattr(const char *path, struct stat *stbuf)
     int res = 0;
 
     memset(stbuf, 0, sizeof(struct stat));
-    if(strcmp(path, "/") == 0) {
+    if (strcmp(path, "/") == 0) {
         stbuf->st_mode = S_IFDIR | 0755;
         stbuf->st_nlink = 2;
-    }
-    else if(strcmp(path, hello_path) == 0) {
+    } else if (strcmp(path, hello_path) == 0) {
         stbuf->st_mode = S_IFREG | 0444;
         stbuf->st_nlink = 1;
         stbuf->st_size = strlen(hello_str);
-    }
-    else
+    } else
         res = -ENOENT;
 
     return res;
@@ -45,7 +43,7 @@ static int hello_readdir(const char *path, void *buf, fuse_fill_dir_t filler,
     (void) offset;
     (void) fi;
 
-    if(strcmp(path, "/") != 0)
+    if (strcmp(path, "/") != 0)
         return -ENOENT;
 
     filler(buf, ".", NULL, 0);
@@ -57,10 +55,10 @@ static int hello_readdir(const char *path, void *buf, fuse_fill_dir_t filler,
 
 static int hello_open(const char *path, struct fuse_file_info *fi)
 {
-    if(strcmp(path, hello_path) != 0)
+    if (strcmp(path, hello_path) != 0)
         return -ENOENT;
 
-    if((fi->flags & 3) != O_RDONLY)
+    if ((fi->flags & 3) != O_RDONLY)
         return -EACCES;
 
     return 0;
