@@ -2,7 +2,7 @@
     FUSE: Filesystem in Userspace
     Copyright (C) 2001-2007  Miklos Szeredi <miklos@szeredi.hu>
 
-    This program can be distributed under the terms of the GNU LGPL.
+    This program can be distributed under the terms of the GNU LGPLv2.
     See the file COPYING.LIB
 */
 
@@ -1425,6 +1425,7 @@ static void fuse_lib_getattr(fuse_req_t req, fuse_ino_t ino,
 
 int fuse_fs_chmod(struct fuse_fs *fs, const char *path, mode_t mode)
 {
+    fuse_get_context()->private_data = fs->user_data;
     if (fs->op.chmod)
         return fs->op.chmod(path, mode);
     else
