@@ -2,7 +2,7 @@
     FUSE: Filesystem in Userspace
     Copyright (C) 2001-2007  Miklos Szeredi <miklos@szeredi.hu>
 
-    This program can be distributed under the terms of the GNU LGPL.
+    This program can be distributed under the terms of the GNU LGPLv2.
     See the file COPYING.LIB
 */
 
@@ -450,6 +450,8 @@ static void do_forget(fuse_req_t req, fuse_ino_t nodeid, const void *inarg)
 
     if (req->f->op.forget)
         req->f->op.forget(req, nodeid, arg->nlookup);
+    else
+        fuse_reply_none(req);
 }
 
 static void do_getattr(fuse_req_t req, fuse_ino_t nodeid, const void *inarg)
