@@ -2,7 +2,7 @@
     FUSE: Filesystem in Userspace
     Copyright (C) 2001-2007  Miklos Szeredi <miklos@szeredi.hu>
 
-    This program can be distributed under the terms of the GNU LGPL.
+    This program can be distributed under the terms of the GNU LGPLv2.
     See the file COPYING.LIB.
 */
 
@@ -14,15 +14,34 @@
 #include <dirent.h>
 #include <errno.h>
 #include <limits.h>
+#include <mntent.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
 
+static int mtab_needs_update(const char *mnt)
+{
+    struct stat stbuf;
+
+    /* If mtab is within new mount, don't touch it */
+    if (strncmp(mnt, _PATH_MOUNTED, strlen(mnt)) == 0 &&
+        _PATH_MOUNTED[strlen(mnt)] == '/')
+        return 0;
+
+    if (lstat(_PATH_MOUNTED, &stbuf) != -1 && S_ISLNK(stbuf.st_mode))
+        return 0;
+
+    return 1;
+}
+
 int fuse_mnt_add_mount(const char *progname, const char *fsname,
                        const char *mnt, const char *type, const char *opts)
 {
     int res;
     int status;
 
+    if (!mtab_needs_update(mnt))
+        return 0;
+
     res = fork();
     if (res == -1) {
         fprintf(stderr, "%s: fork: %s\n", progname, strerror(errno));
@@ -72,6 +91,9 @@ int fuse_mnt_umount(const char *progname, const char *mnt, int lazy)
     int res;
     int status;
 
+    if (!mtab_needs_update(mnt))
+        return 0;
+
     res = fork();
     if (res == -1) {
         fprintf(stderr, "%s: fork: %s\n", progname, strerror(errno));
